from matplotlib import pyplot as plt
from matplotlib.colors import ListedColormap
import numpy as np

def cseis():
	seis=np.concatenate(
(np.concatenate((0.5*np.ones([1,40]),np.expand_dims(np.linspace(0.5,1,88),axis=1).transpose(),np.expand_dims(np.linspace(1,0,88),axis=1).transpose(),np.zeros([1,40])),axis=1).transpose(),
np.concatenate((0.25*np.ones([1,40]),np.expand_dims(np.linspace(0.25,1,88),axis=1).transpose(),np.expand_dims(np.linspace(1,0,88),axis=1).transpose(),np.zeros([1,40])),axis=1).transpose(),
np.concatenate((np.zeros([1,40]),np.expand_dims(np.linspace(0,1,88),axis=1).transpose(),np.expand_dims(np.linspace(1,0,88),axis=1).transpose(),np.zeros([1,40])),axis=1).transpose()),axis=1)

	return ListedColormap(seis)
	

def plot3d(d3d,frames=None,z=None,x=None,y=None,figname=None,showf=True,**kwargs):
	'''
	plot3d: plot beautiful 3D slices
	
	By Yangkang Chen
	June, 18, 2023
	
	EXAMPLE 1
	import numpy as np
	d3d=np.random.rand(100,100,100);
	from pyseistr import plot3d
	plot3d(d3d);
	
	EXAMPLE 2
	import scipy
	data=scipy.io.loadmat('/Users/chenyk/chenyk/matlibcyk/test/hyper3d.mat')['cmp']
	from pyseistr import plot3d
	plot3d(data);
	'''

	[nz,nx,ny] = d3d.shape;
	
	if frames is None:
		frames=[int(nz/2),int(nx/2),int(ny/2)]
		
	X, Y, Z = np.meshgrid(np.arange(nx)*0.01, np.arange(ny)*0.01, np.arange(nz)*0.01)

	d3d=d3d.transpose([1,2,0])
	
	kw = {
    'vmin': d3d.min(),
    'vmax': d3d.max(),
    'levels': np.linspace(d3d.min(), d3d.max(), 100),
    'cmap':cseis()
	}
	kw.update(kwargs)
	
	fig = plt.figure(figsize=(8, 8))
	
	ax = fig.add_subplot(111, aspect='auto',projection='3d')
	plt.jet()

	# Plot contour surfaces
	_ = ax.contourf(
	X[:, :, -1], Y[:, :, -1], d3d[:, :, frames[0]].transpose(), #x,y,z
	zdir='z', offset=0, alpha=1, **kw
	)

	_ = ax.contourf(
	X[0, :, :], d3d[:, frames[2], :], Z[0, :, :],
	zdir='y', offset=0, alpha=1, **kw
	)
	C = ax.contourf(
	d3d[frames[1], :, :], Y[:, -1, :], Z[:, -1, :],
	zdir='x', offset=X.max(), alpha=1.0, **kw
	)

	plt.gca().set_xlabel("X",fontsize='large', fontweight='normal')
	plt.gca().set_ylabel("Y",fontsize='large', fontweight='normal')
	plt.gca().set_zlabel("Z",fontsize='large', fontweight='normal')

	xmin, xmax = X.min(), X.max()
	ymin, ymax = Y.min(), Y.max()
	zmin, zmax = Z.min(), Z.max()
	ax.set(xlim=[xmin, xmax], ylim=[ymin, ymax], zlim=[zmin, zmax])
	plt.gca().invert_zaxis()
	
	if figname is not None:
		plt.savefig(figname,**kwargs)
	
	if showf:
		plt.show()
	else:
		plt.close() #or plt.clear() ?
		
		

	
	


import numpy as np
from soint2dcfun import *
from soint3dcfun import *

def sint2d(din,mask,dip,niter=100,eps=0.01,ns=1,order=1,verb=1):
	'''
	
	sint2d: Interpolation for sparse data in 2D, e.g., well logs

	Yangkang Chen
	Aug, 11, 2021
	Ported to Python in Jun, 15, 2023
	
	INPUT
	din:
	dip:
	mask:
	eps:
	ns:
	order:
	
	OUTPUT
	dout
	
	EXAMPLE
	
	
	'''
	from .solvers import conjgrad
	from .pwsmooth import pwsmooth_set,pwsmooth_lop
	from .operators import mask_lop
	
	[n1,n2]=din.shape;
	n12=n1*n2;
	mm=din.flatten(order='F'); #din is a 2D matrix
	mask=mask.flatten(order='F');

	#figure out scaling and make known data mask
	lam=0;
	for ii in range(n12):
		if mask[ii] !=0:
			lam=lam+1;
	lam=np.sqrt(lam/n12);
	
	w1 = pwsmooth_set(dip,n1,n2,ns,order,eps);
	par_L={'nm':n12,'nd':n12,'mask':mask}
	par_S={'nm':n12,'nd':n12,'dip':dip,'w1':w1,'ns':ns,'order':order,'eps':eps}
	
	eps_cg=lam*lam;
	tol_cg=0.0000019209;
	ifhasp0=1;
	
	# Conjugate Gradient with Shaping
	mm = conjgrad(None, mask_lop, pwsmooth_lop, mm, mm, mm, eps_cg, tol_cg, niter,ifhasp0,[],par_L,par_S,verb);
	dout=mm.reshape(n1,n2,order='F')
	
	return dout
	
	
	
def sint2dc(din,mask,dip,niter=100,eps=0.01,ns=1,order=1,verb=1):
	'''
	
	sint2dc: Interpolation for sparse data in 2D, e.g., well logs

	Yangkang Chen
	Aug, 11, 2021
	Ported to Python in Jun, 15, 2023
	
	INPUT
	din:
	dip:
	mask:
	eps:
	ns:
	order:
	
	OUTPUT
	dout
	
	EXAMPLE
	
	
	'''
	
	[n1,n2]=din.shape;	
	din=np.float32(din).flatten(order='F');
	mask=np.float32(mask).flatten(order='F');
	dip=np.float32(dip).flatten(order='F');
	
	dout=csint2d(din,dip,mask,n1,n2,niter,ns,order,verb,eps);
	dout=dout.reshape(n1,n2,order='F');
	
	return dout	
	
	
def sint3dc(din,mask,dipi,dipx,niter=100,eps=0.01,ns1=1,ns2=1,order1=1,order2=1,verb=1):
	'''
	
	sint3dc: Interpolation for sparse data in 2D, e.g., well logs

	Yangkang Chen
	Aug, 11, 2021
	Ported to Python in Jun, 15, 2023
	
	INPUT
	din:
	mask:
	dip:
	eps:
	ns:
	order:
	
	OUTPUT
	dout
	
	EXAMPLE
	
	
	'''
	
	[n1,n2,n3]=din.shape;	
	din=np.float32(din).flatten(order='F');
	mask=np.float32(mask).flatten(order='F');
	dipi=np.float32(dipi).flatten(order='F');
	dipx=np.float32(dipx).flatten(order='F');
	
	dout=csint3d(din,dipi,dipx,mask,n1,n2,n3,niter,ns1,ns2,order1,order2,verb,eps);
	dout=dout.reshape(n1,n2,n3,order='F');
	
	return dout	



def somean2d(dn,dip,ns,order,eps):
	'''
	somean2d: plane-wave smoothing 
	
	INPUT
	dn: model   noisy data
	dip: slope (2D array)
	ns:       spray radius
	order:    PWD order
	eps: regularization (default:0.01);

	OUTPUT
	ds:  smoothed data
	
	EXAMPLE
	
	
	'''
	import numpy as np
	n1=dn.shape[0];
	n2=dn.shape[1];

	ns2=2*ns+1;	#spray diameter

	#flattening
	from .pwspray2d import pwspray2d
	
	utmp=pwspray2d(dn,dip,ns,order,eps);
	u=utmp.reshape(n1,ns2,n2,order='F')
	ds=np.sum(u,1)/ns2;

	return ds




import numpy as np
def pwsmooth_set(dip,n1,n2,ns,order,eps):
	'''
	pwsmooth_set: set local slope for pwsmooth_lop
	
	INPUT
	dip:
	n1/n2:
	ns:
	order:
	eps:
	
	OUTPUT
	w1: 2D matrix [n1,n2]
	'''
	n12=n1*n2;
	w1=np.ones(n12);
	
	par={'nm':n12,'nd':n12,'dip':dip,'w1':w1.reshape(n1,n2,order='F'),'ns':ns,'order':order,'eps':eps}
	t=pwsmooth_lop(w1,par,False,False);
	for i1 in range(n12):
		if t[i1] != 0.0:
			w1[i1]=1.0/t[i1];
		else:
			w1[i1]=0.0;
	w1=w1.reshape(n1,n2,order='F');
	
	return w1
	
	
def pwsmooth_lop(din,par,adj,add):
	'''
	pwsmooth_lop: plane-wave smoothing operator
	BY Yangkang Chen, Aug, 10, 2021
	
	INPUT
	d: model/data
	par: parameter (dip,w1,ns,order,eps,nm,nd)
	adj: adj flag
	add: add flag
	
	OUTPUT
	m: data/model
	'''
	from .operators import adjnull
	from .pwspray2d import pwspray_lop
	
	dip=par['dip'];
	w1=par['w1'];
	[n1,n2]=dip.shape;
	ns=par['ns'];
	order=par['order'];
	eps=par['eps'];
	nm=par['nm']
	nd=par['nd']
	ndn=n1*n2;
	nds=n1*n2;

	if adj==1:
		d=din;
		if 'm' in par and add==1:
			m=par['m'];
		else:
			m=np.zeros(par['nm']);
	else:
		m=din;
		if 'd' in par and add==1:
			d=par['d'];
		else:
			d=np.zeros(par['nd']);

	dn=m;
	ds=d;
	
	if ndn!=nds:
		print('Wrong size %d = %d'%(ndn,nds))

	[ dn,ds ] = adjnull( adj,add,ndn,nds,dn,ds );

	ns2=2*ns+1;#spray diameter
	n12=n1*n2;

	u=np.zeros([n1,ns2,n2]);
	utmp=np.zeros(n1*n2*ns2);
	w=np.zeros(ns2);

	for iss in range(ns2):
		w[iss]=ns+1-abs(iss-ns);

	#for Normalization
	#t=zeros(n12,1);
	
	par.update({'nt':n1,'nx':n2,'nm':n1*n2,'nd':n1*n2*ns2})

	if adj:
		for i2 in range(n2):
			for i1 in range(n1):
				ws=w1[i1,i2];
				for iss in range(ns2):
					u[i1,iss,i2]=ds[i2*n1+i1]*w[iss]*ws;
	
		utmp=u.flatten(order='F');
		dn=pwspray_lop(utmp,par,True,True);
	else:
		utmp=pwspray_lop(dn,par,False,False);
		u=utmp.reshape(n1,ns2,n2,order='F');
		for i2 in range(n2):
			for i1 in range(n1):
				ws=w1[i1,i2]
				for iss in range(ns2):
					ds[i2*n1+i1]=ds[i2*n1+i1]+u[i1,iss,i2]*w[iss]*ws;
	
	if adj==1:
		dout=dn;
	else:
		dout=ds;
		
	return dout
	
	


import numpy as np
def gensyn(noise=False,seed=202122,var=0.2):
	'''
	gensyn: quickly generate the representative synthetic data used in the paper
	
	INPUT
	noise: if add noise
	seed: random number seed
	var: noise variance (actually the maximum amplitude of noise)
	
	OUTPUT
	case 1. data: clean synthetic data
	case 2. data,noisy: noisy case
		
	EXAMPLE 1
	from pyseistr import gensyn
	data=gensyn();
	import matplotlib.pyplot as plt;
	plt.imshow(data);plt.ylabel('Time sample');plt.xlabel('Trace');plt.show();

	EXAMPLE 2
	from pyseistr import gensyn
	data,noisy=gensyn(noise=True);
	import matplotlib.pyplot as plt;
	plt.subplot(1,2,1);plt.imshow(data,clim=[-0.2,0.2],aspect='auto');plt.xlabel('Trace');plt.ylabel('Time sample');
	plt.subplot(1,2,2);plt.imshow(noisy,clim=[-0.2,0.2],aspect='auto');plt.xlabel('Trace');
	plt.show();
	
	'''
	import numpy as np
	from .ricker import ricker
	[w,tw]=ricker(30,0.001,0.1);
	t=np.zeros([300,1000]);sigma=300;A=100;B=200;
	data=np.zeros([400,1000]);

	[m,n]=t.shape;
	for i in range(1,n+1):
		k=np.floor(-A*np.exp(-np.power(i-n/2,2)/np.power(sigma,2))+B);k=int(k);
		if k>1 and k<=m:
			t[k-1,i-1]=1;

	for i in range(1,n+1):
		data[:,i-1]=np.convolve(t[:,i-1],w);
	
	if noise:
		data=data/np.max(np.max(data));
		np.random.seed(seed);
		noise=(np.random.rand(data.shape[0],data.shape[1])*2-1)*var;
		noisy=data+noise
		return data,noisy
	else:
		return data
	
	
def genflat(nt=100,nx=60,t=[30,50,80],amp=[1,1,1],freq=30,dt=0.004):
	'''
	genflat: quickly generate the representative synthetic data used in the paper
	
	INPUT
	None
	
	OUTPUT
	data: synthetic data
	
	EXAMPLE
	from pyseistr import genflat
	data=genflat();
	import matplotlib.pyplot as plt;
	plt.imshow(data);plt.ylabel('Time sample');plt.xlabel('Trace');plt.show();

	'''
	import numpy as np
	from .ricker import ricker
	[w,tw]=ricker(freq,dt,60*dt);
	data=np.zeros([nt,nx]);

	trace=np.zeros(nt)
	for ii in range(len(t)):
		trace[t[ii]]=amp[ii];

	for ii in range(nx):
		data[:,ii]=np.convolve(trace,w,mode='same');
	
	return data
	
	
def sigmoid(n1=400,n2=100,d1=0.004,d2=0.032,o1=0,o2=0,large=None,reflectivity=True,taper=True,sd=19921995):
	'''
	sigmoid: generate the famous Sigmoid synthetic data
	
	INPUT
	n1,n2: 			dimension
	large:			length of reflectivity series
	reflectivity: 	if output reflectivity (otherwise output impedance model)
	taper: 			if taper the edges
	
	sd: random number seed
	
	o1,o2,d1,d2: actually no use (just for retaining the previous parameters)
	
	OUTPUT

	RFERENCE
	October 2014 program of the month:
	https://reproducibility.org/blog/2014/10/08/program-of-the-month-sfsigmoid/
	
	EXAMPLE 1
	from pyseistr import sigmoid
	data=sigmoid();
	import matplotlib.pyplot as plt;
	plt.imshow(data,aspect='auto');plt.ylabel('Time sample');plt.xlabel('Trace');plt.show();

	EXAMPLE 2
	from pyseistr import sigmoid
	data=sigmoid(sd=2024);
	import matplotlib.pyplot as plt;
	plt.imshow(data,aspect='auto');plt.ylabel('Time sample');plt.xlabel('Trace');plt.show();
	
	EXAMPLE 3 (set "static long int seed = 1996;" in "RSFSRC/user/gee/random.c")
	from pyseistr import sigmoid
	data=sigmoid(n1=200,n2=210);
	import m8r,os,numpy as np
	os.system('sfsigmoid n1=200 n2=210 d2=0.008>tt.rsf')
	data2 = m8r.File('tt.rsf')[:].transpose()
	print('Difference between pyseistr and Madagascar:',np.linalg.norm(data-data2))
	
	import matplotlib.pyplot as plt;
	plt.figure(figsize=(12,8))
	plt.subplot(1,3,1);
	plt.imshow(data,clim=(-0.05,0.05),aspect='auto');plt.xlabel('Trace');plt.ylabel('Time sample');plt.title('pyseistr');
	plt.subplot(1,3,2);
	plt.imshow(data2,clim=(-0.05,0.05),aspect='auto');plt.xlabel('Trace');plt.title('Madagascar');
	plt.subplot(1,3,3);
	plt.imshow(data-data2,clim=(-0.05,0.05),aspect='auto');plt.xlabel('Trace');plt.title('Difference');
	plt.show();

	'''
	if large is None:
		large=5*n1;
	
	imp1=np.zeros(large);
	imp2=np.zeros(large);
	dipper=np.zeros([n1,n2]);
	earth=np.zeros([n1,n2]);
	refl=np.zeros([n1,n2]);
	sig1=np.zeros([n1,n2]);
	sig2=np.zeros([n1,n2]);
	fault=np.zeros([n1,n2]);
	
	#seed for the random number generator
# 	sd=19921995;
	
	imp1[0]=1.0;
	for i1 in range(1,large,1):
		rand,sd=random0(seed=sd);
		if rand>0.2:
			imp1[i1]=imp1[i1-1];
		else:
			rand,sd=random0(seed=sd);
			imp1[i1]=1.0+0.1*rand;
		
	imp2[0]=1.0;
	for i1 in range(1,large,1):
		rand,sd=random0(seed=sd);
		if rand>0.3:
			imp2[i1]=imp2[i1-1];
		else:
			rand,sd=random0(seed=sd);
			imp2[i1]=1.0+0.1*rand;
			
	for i2 in range(n2):
		for i1 in range(n1):
			t = i1 + (i2+1.) * 0.3 * (1.0*n1)/(1.0*n2);
			it=int(t);
			frac=t-it;
			
			if it>=0 and it+1<large:
				dipper[i1,i2]=imp1[it]*(1-frac)+imp1[it+1]*frac;
			else:
				dipper[i1,i2]=0;
				
	
	for i2 in range(n2):
		for i1 in range(n1):
			t = i1 + n1 *0.15 * ( 1.1 + np.cos( ( 3 *3.14 * (i2+1.))/n2) );
			it=int(t);
			iss=it+10;
			frac=t-it;
			
			if it>=0 and it+1<large and iss>=0 and iss+1<large:
				sig1[i1,i2]=imp2[it]*(1-frac)+imp2[it+1]*frac;
				sig2[i1,i2]=imp2[iss]*(1-frac)+imp2[iss+1]*frac;
			else:
				sig1[i1,i2]=0;
				sig2[i1,i2]=0;
			
	for i2 in range(n2):
		for i1 in range(n1):
			if (i2+1.0)/n2-0.9 < -0.75*np.power((i1+2.0)/n1,2):
				fault[i1,i2]=sig1[i1,i2];
			else:
				fault[i1,i2]=sig2[i1,i2];
		
	for i2 in range(n2):
		for i1 in range(n1):
			it = int(i1 + 1. + (i2 + 1.) * .3 * (1.*n1)/(1.*n2));
			if it>n1/2:
				earth[i1,i2]=fault[i1,i2];
			else:
				earth[i1,i2]=dipper[i1,i2];
	
	if reflectivity:
		for i2 in range(n2):
			refl[0,i2]=0.0;
			for i1 in range(1,n1,1):
				refl[i1,i2] = (earth[i1-1,i2]-earth[i1,i2])/(earth[i1-1,i2]+earth[i1,i2])
		
		if taper:
			for i2 in range(10):
				for i1 in range(n1):
					refl[i1,i2]=refl[i1,i2]*(i2/10.0);
					refl[i1,n2-i2-1]=refl[i1,n2-i2-1]*(i2/10.0);
		
			for i2 in range(5):
				for i1 in range(n1):
					refl[i1,i2]=refl[i1,i2]*(i2/5.0);
					refl[i1,n2-i2-1]=refl[i1,n2-i2-1]*(i2/5.0);
					
		return refl
	else:
		return earth
	

def random0(seed=1996,ia=727,im=524287):
	'''
	random0: Simple pseudo-random number generator
	'''
	seed = np.mod(int(seed*ia),im)
	rand = (float(seed)-0.5)/(float(im-1));
	return rand,seed
	







from paint2dcfun import *
import numpy as np

def rgt(dip,o1=0,d1=0.004,order=1,i0=0,eps=0.01,verb=False):
	'''
	rgt: Generate relative geological time
	
	INPUT
	dip: 	slope
	o1: 	starting time in axis 0
	d1: 	time interval in axis 0
	
	OUTPUT
	time: 	relative geological time
	
	EXAMPLE 1
	from pyseistr import smooth
	from pyseistr import dip2dc
	from pyseistr import rgt
	import matplotlib.pyplot as plt;
	from pyseistr import sigmoid
	sig=sigmoid(n1=200,n2=210);
	sig=smooth(sig,rect=[3,1,1],diff=[1,0,0],adj=0);
	sig=smooth(sig,rect=[3,1,1])
	sdip=dip2dc(sig,order=2,niter=10,rect=[4,4,1],verb=0)
	time=rgt(sdip,o1=0,d1=0.004,order=2,i0=50,eps=0.1);

	fig = plt.figure(figsize=(12, 6))
	ax=plt.subplot(1,3,1)
	plt.imshow(sig,cmap='gray',clim=(-0.02, 0.02),aspect='auto');ax.set_xticks([]);ax.set_yticks([]);plt.title('Seismic');
	plt.colorbar(orientation='horizontal');
	ax=plt.subplot(1,3,2)
	plt.imshow(sdip,cmap='jet',clim=(-2, 2),aspect='auto');ax.set_xticks([]);ax.set_yticks([]);plt.title('Slope');
	plt.colorbar(orientation='horizontal');
	ax=plt.subplot(1,3,3)
	plt.imshow(time,cmap='jet',clim=(0, 0.7),aspect='auto');ax.set_xticks([]);ax.set_yticks([]);plt.title('RGT');
	plt.colorbar(orientation='horizontal');
	plt.show()
	
	EXAMPLE 2
	from pyseistr import rgt
	from pylib.io import binread
	import matplotlib.pyplot as plt;
	dip=binread('/Users/chenyk/data/datapath/tccs/flat/flat/sdip-pad.rsf@',n1=300,n2=210).copy(order='F')
	time=rgt(dip,o1=-0.2,d1=0.004,order=2,i0=105,eps=0.1);
	plt.imshow(dip,cmap='jet');plt.colorbar();plt.show();
	plt.imshow(time,cmap='jet');plt.colorbar();plt.show();
	
	'''
	[n1,n2]=dip.shape;
	trace=np.linspace(0,d1*(n1-1),n1)+o1;
	time=pwpaintc(dip,trace,order=order,i0=i0,eps=eps,verb=verb);
	
	return time
	
	
def pwpaintc(dip,trace,order=1,i0=0,eps=0.01,verb=False):
	'''
	pwpaint: plane-wave painting (C version)
	
	INPUT
	dip:		slope
	trace:		seed trace (this can also be a time trace for RGT)
	order:		accuracy order 
	eps:		regularization 
	io:			reference trace
	
	OUTPUT
	dout:		painted gather
	
	EXAMPLE
	from pyseistr import pwpaintc
	from pylib.io import binread
	import matplotlib.pyplot as plt;
	dip=binread('/Users/chenyk/data/datapath/tccs/flat/flat/sdip-pad.rsf@',n1=300,n2=210).copy(order='F')
	trace=binread('/Users/chenyk/data/datapath/tccs/flat/flat/trace.rsf@',n1=300,n2=1)
	dout=pwpaintc(dip,trace,order=2,i0=105,eps=0.1);
	plt.imshow(dip);plt.show();
	plt.plot(trace);plt.show();
	plt.imshow(dout);plt.show();
	'''

	[n1,n2]=dip.shape;
	dip=np.float32(dip).flatten(order='F');
	trace=np.float32(trace).flatten(order='F');
	dout=cpaint2d(dip,trace,n1,n2,order,i0,eps,verb);
	dout=dout.reshape(n1,n2,order='F');
	
	return dout
	
	


import numpy as np
from dipcfun import *

def smooth(din,rect=[1,1,1],diff=[0,0,0],box=[0,0,0],repeat=1,adj=1):
	'''
	
	INPUT
	din:	input noisy data
	rect: 	smoothing radius on #-th axis
	diff:	differentiation on #-th axis (default: 0)
	box:	box (rather than triangle) on #-th axis )
	repeat:	repeat filtering several times
	adj:	run in the adjoint mode 
	
	OUTPUT
	dout:	smoothed data
	
	EXAMPLE 1
	from pyseistr import smooth
	from pyseistr import gensyn
	import matplotlib.pyplot as plt;
	
	data,noisy=gensyn(noise=True);
	data=data[:,0::10];noisy=noisy[:,0::10];
	data1=smooth(noisy,rect=[1,5,1]);
	
	fig = plt.figure(figsize=(10, 8))
	ax=plt.subplot(1,4,1)
	plt.imshow(data,cmap='jet',clim=(-0.2, 0.2),aspect='auto');ax.set_xticks([]);ax.set_yticks([]);plt.title('Clean data');
	ax=plt.subplot(1,4,2)
	plt.imshow(noisy,cmap='jet',clim=(-0.2, 0.2),aspect='auto');ax.set_xticks([]);ax.set_yticks([]);plt.title('Noisy data');
	ax=plt.subplot(1,4,3)
	plt.imshow(data1,cmap='jet',clim=(-0.2, 0.2),aspect='auto');ax.set_xticks([]);ax.set_yticks([]);plt.title('Smoothed data');
	ax=plt.subplot(1,4,4)
	plt.imshow(noisy-data1,cmap='jet',clim=(-0.2, 0.2),aspect='auto');ax.set_xticks([]);ax.set_yticks([]);plt.title('Noise');
	plt.show()
	
	EXAMPLE 2 (set "static long int seed = 1996;" in "RSFSRC/user/gee/random.c")
	from pyseistr import sigmoid
	from pyseistr import smooth
	data=sigmoid(n1=200,n2=210);
	data1=smooth(data,rect=[3,3,1],diff=[1,0,0],adj=1);
	
	import m8r,os,numpy as np
	os.system('sfsigmoid n1=200 n2=210 d2=0.008>tt.rsf')
	os.system('sfsmooth<tt.rsf rect1=3 rect2=3 diff1=1 adj=y >tt2.rsf')
	data2 = m8r.File('tt2.rsf')[:].transpose()
	print('Difference between pyseistr and Madagascar:',np.linalg.norm(data1-data2))
	
	import matplotlib.pyplot as plt;
	plt.figure(figsize=(12,8))
	plt.subplot(1,3,1);
	plt.imshow(data1,clim=(-0.05,0.05),aspect='auto');plt.xlabel('Trace');plt.ylabel('Time sample');plt.title('pyseistr');
	plt.subplot(1,3,2);
	plt.imshow(data2,clim=(-0.05,0.05),aspect='auto');plt.xlabel('Trace');plt.title('Madagascar');
	plt.subplot(1,3,3);
	plt.imshow(data1-data2,clim=(-0.05,0.05),aspect='auto');plt.xlabel('Trace');plt.title('Difference');
	plt.show();
	
	'''
	from .divne import smooth as smooth1
	from .divne import triangle_init,first_index,smooth2

	if din.ndim==2:	#for 2D problems
		din=np.expand_dims(din, axis=2)
	
	[n1,n2,n3]=din.shape;
	nrep=repeat;

	if n3>1:
		dim=3;
	else:
		dim=2;

	if rect[2]>1:
		dim1=2;
	else:
		if rect[1]>1:
			dim1=1;
		else:
			dim1=0;

	n=[n1,n2,n3];
	s=[1,n1,n1*n2];

	if dim1==0:
		n2=n2*n3;
	else:
		if dim1==1:
			n1=n1*n2;n2=n3;
		else:
			n1=n1*n2*n3;n2=1;

	din=din.flatten(order='F');
	dout=[]
	for i2 in range(0,n2):
		x=din[i2*n1:(i2+1)*n1];
		for i in range(dim1+1):
			if (rect[i] <= 1):
				continue;
			tr = triangle_init (rect[i],n[i],box[i]);
			for j in range(0,int(n1/n[i])):
				i0 = first_index (i,j,dim1+1,n,s);
				for irep in range(nrep):
					if adj:
						x,tr=smooth1 (tr,i0,s[i],diff[i],x);
					else:
						x,tr=smooth2 (tr,i0,s[i],diff[i],x);
		dout.append(x)
	dout=np.concatenate(dout,axis=0).reshape(n[0],n[1],n[2],order='F')
	dout=np.squeeze(dout);
	return dout
	
	
def smoothc(din,rect=[1,1,1],diff=[0,0,0],box=[0,0,0],repeat=1,adj=1):
	'''
	
	INPUT
	din:	input noisy data
	rect: 	smoothing radius on #-th axis
	diff:	differentiation on #-th axis (default: 0)
	box:	box (rather than triangle) on #-th axis )
	repeat:	repeat filtering several times
	adj:	run in the adjoint mode 
	
	OUTPUT
	dout:	smoothed data
	
	EXAMPLE 1
	from pyseistr import smoothc
	from pyseistr import gensyn
	import matplotlib.pyplot as plt;
	
	data,noisy=gensyn(noise=True);
	data=data[:,0::10];noisy=noisy[:,0::10];
	data1=smoothc(noisy,rect=[1,5,1]);
	
	fig = plt.figure(figsize=(10, 8))
	ax=plt.subplot(1,4,1)
	plt.imshow(data,cmap='jet',clim=(-0.2, 0.2),aspect='auto');ax.set_xticks([]);ax.set_yticks([]);plt.title('Clean data');
	ax=plt.subplot(1,4,2)
	plt.imshow(noisy,cmap='jet',clim=(-0.2, 0.2),aspect='auto');ax.set_xticks([]);ax.set_yticks([]);plt.title('Noisy data');
	ax=plt.subplot(1,4,3)
	plt.imshow(data1,cmap='jet',clim=(-0.2, 0.2),aspect='auto');ax.set_xticks([]);ax.set_yticks([]);plt.title('Smoothed data');
	ax=plt.subplot(1,4,4)
	plt.imshow(noisy-data1,cmap='jet',clim=(-0.2, 0.2),aspect='auto');ax.set_xticks([]);ax.set_yticks([]);plt.title('Noise');
	plt.show()
	
	EXAMPLE 2 (set "static long int seed = 1996;" in "RSFSRC/user/gee/random.c")
	from pyseistr import sigmoid
	from pyseistr import smoothc
	data=sigmoid(n1=200,n2=210);
	data1=smoothc(data,rect=[3,3,1],diff=[1,0,0],adj=1);
	
	import m8r,os,numpy as np
	os.system('sfsigmoid n1=200 n2=210 d2=0.008>tt.rsf')
	os.system('sfsmooth<tt.rsf rect1=3 rect2=3 diff1=1 adj=y >tt2.rsf')
	data2 = m8r.File('tt2.rsf')[:].transpose()
	print('Difference between pyseistr and Madagascar:',np.linalg.norm(data1-data2))
	
	import matplotlib.pyplot as plt;
	plt.figure(figsize=(12,8))
	plt.subplot(1,3,1);
	plt.imshow(data1,clim=(-0.05,0.05),aspect='auto');plt.xlabel('Trace');plt.ylabel('Time sample');plt.title('pyseistr');
	plt.subplot(1,3,2);
	plt.imshow(data2,clim=(-0.05,0.05),aspect='auto');plt.xlabel('Trace');plt.title('Madagascar');
	plt.subplot(1,3,3);
	plt.imshow(data1-data2,clim=(-0.05,0.05),aspect='auto');plt.xlabel('Trace');plt.title('Difference');
	plt.show();
	'''
	
	if din.ndim==2:	#for 2D problems
		din=np.expand_dims(din, axis=2)
	
	[n1,n2,n3]=din.shape;
	r1,r2,r3=rect;
	diff1,diff2,diff3=diff;
	box1,box2,box3=box;
	
	din=np.float32(din.flatten(order='F'));
	dout=smoothcf(din,n1,n2,n3,repeat,adj,r1,r2,r3,diff1,diff2,diff3,box1,box2,box3);
	dout=np.squeeze(dout.reshape(n1,n2,n3,order='F'));
	
	return dout